package com.gic.auth.service.impl;

import com.gic.auth.dao.mapper.TabAuditorAuditedGroupRelMapper;
import com.gic.auth.dto.AuditorAuditedGroupRelDTO;
import com.gic.auth.entity.TabAuditedGroupUserRel;
import com.gic.auth.entity.TabAuditorAuditedGroupRel;
import com.gic.auth.service.AuditedGroupUserRelService;
import com.gic.auth.service.AuditorAuditedGroupRelService;
import com.gic.commons.util.GlobalInfo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-18 16:30
 */
@Service
public class AuditorAuditedGroupRelServiceImpl implements AuditorAuditedGroupRelService {

    @Autowired
    private TabAuditorAuditedGroupRelMapper tabAuditorAuditedGroupRelMapper;
    @Autowired
    private AuditedGroupUserRelService auditedGroupUserRelService;

    @Override
    public int delRel(Integer auditedGroupId) {
        return tabAuditorAuditedGroupRelMapper.delRel(auditedGroupId);
    }

    @Override
    public void addOrUpdateRel(Integer enterpriseId, Integer auditorId, List<Integer> auditedGroupIdList) {
        // 把所有auditorId的关联status设置为0
        tabAuditorAuditedGroupRelMapper.delByAuditorId(auditorId);
        // 插入不存在的关联
        for (Integer auditedGroupId : auditedGroupIdList) {
            TabAuditorAuditedGroupRel itemRel = new TabAuditorAuditedGroupRel();
            itemRel.setEnterpriseId(enterpriseId);
            itemRel.setAuditorId(auditorId);
            itemRel.setAuditedGroupId(auditedGroupId);
            itemRel.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
            itemRel.setCreateTime(new Date());
            tabAuditorAuditedGroupRelMapper.insertSelectiveByNotExist(itemRel);
        }
        // 把所有auditedGroup关联status都设置为1
        tabAuditorAuditedGroupRelMapper.updateStatusByAuditedGroupIds(enterpriseId, auditorId, auditedGroupIdList);
    }

    @Override
    public void delRelByAuditorId(Integer auditorId) {
        tabAuditorAuditedGroupRelMapper.delByAuditorId(auditorId);
    }

    @Override
    public List<TabAuditorAuditedGroupRel> listRelByAuditorId(Integer auditorId) {
        AuditorAuditedGroupRelDTO auditorAuditedGroupRel = new AuditorAuditedGroupRelDTO();
        auditorAuditedGroupRel.setAuditorId(auditorId);
        return tabAuditorAuditedGroupRelMapper.listRel(auditorAuditedGroupRel);
    }

    @Override
    public Map<Integer, Integer> getUserCountByAuditorIds(List<Integer> auditorIdList) {
//      getAuditedGroupBelongAuditor
        AuditorAuditedGroupRelDTO auditorAuditedGroupRelDTO = new AuditorAuditedGroupRelDTO();
        auditorAuditedGroupRelDTO.setAuditorIdList(auditorIdList);
        List<TabAuditorAuditedGroupRel> relList = tabAuditorAuditedGroupRelMapper.listRel(auditorAuditedGroupRelDTO);
        if (CollectionUtils.isEmpty(relList)) {
            return Collections.emptyMap();
        }
        // 根据受审组查询受审组对应的用户
        List<Integer> auditedGroupIds = relList.stream().map(TabAuditorAuditedGroupRel::getAuditedGroupId).collect(Collectors.toList());
        List<TabAuditedGroupUserRel> userList = auditedGroupUserRelService.getUserIdListByAuditedGroup(auditedGroupIds);
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        // 获取 受审组 -> 审核员 的map
        Map<Integer, Set<Integer>> groupAuditorMap = relList.stream().collect(Collectors.groupingBy(TabAuditorAuditedGroupRel::getAuditedGroupId, Collectors.mapping(TabAuditorAuditedGroupRel::getAuditorId, Collectors.toSet())));
        Map<Integer, Set<Integer>> tmpMap = new HashMap<>();
        for (TabAuditedGroupUserRel userRel : userList) {
            Integer auditedGroupId = userRel.getAuditedGroupId();
            Set<Integer> auditorIdSet = groupAuditorMap.get(auditedGroupId);
            for (Integer auditorId : auditorIdSet) {
                Set<Integer> set = tmpMap.computeIfAbsent(auditorId, k -> new HashSet<>());
                set.add(userRel.getUserId());
            }
        }
        Map<Integer, Integer> resultMap = new HashMap<>();
        for (Map.Entry<Integer, Set<Integer>> entry : tmpMap.entrySet()) {
            Set<Integer> set = entry.getValue();
            resultMap.put(entry.getKey(), set == null ? 0 : set.size());
        }
        return resultMap;
    }

    @Override
    public List<Integer> listRelByAuditedGroupIdAndAuditId(Integer enterpriseId, List<Integer> auditedGroupIds, List<Integer> projectAuditorIds) {
        return tabAuditorAuditedGroupRelMapper.listRelByAuditedGroupIdAndAuditId(enterpriseId, auditedGroupIds, projectAuditorIds);
    }
}
