package com.gic.auth.service.impl;

import com.gic.auth.constant.LoginUserTypeEnum;
import com.gic.auth.dao.mapper.TabSysUserMapper;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserListDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author guojx
 * @date 2019/7/16 6:52 PM
 */
@Service("userService")
public class UserServiceImpl implements UserService {
    @Autowired
    private TabSysUserMapper tabSysUserMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveUser(UserDTO userDTO) {
        userDTO.setLoginType(LoginUserTypeEnum.GIC_USER.getCode());
        userDTO.setCreateTime(new Date());
        userDTO.setUpdateTime(new Date());
        userDTO.setStatus(1);
        TabSysUser tabUser = EntityUtil.changeEntityNew(TabSysUser.class, userDTO);
        tabSysUserMapper.insert(tabUser);
        return tabUser.getUserId();
    }

    @Override
    public int saveOperationUser(UserDTO userDTO) {
        userDTO.setLoginType(LoginUserTypeEnum.OPERATION_USER.getCode());
        userDTO.setCreateTime(new Date());
        userDTO.setUpdateTime(new Date());
        userDTO.setStatus(1);
        //设置超管权限
        userDTO.setSuperAdmin(1);
        TabSysUser tabUser = EntityUtil.changeEntityNew(TabSysUser.class, userDTO);
        tabSysUserMapper.insertSelective(tabUser);
        return tabUser.getUserId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void editUser(UserDTO userDTO) {
        TabSysUser tabUser = EntityUtil.changeEntityNew(TabSysUser.class, userDTO);
        tabUser.setUpdateTime(new Date());
        tabSysUserMapper.updateByPrimaryKeySelective(tabUser);
    }

    @Override
    public TabSysUser getUserById(Integer userId) {
        return tabSysUserMapper.selectByPrimaryKey(userId);
    }

    @Override
    public boolean isPhoneRepeat(Integer userId, String phone, Integer enterpriseId) {
        int count = tabSysUserMapper.countByPhone(phone, userId, enterpriseId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public boolean isPhoneRepeat(Integer userId, String phone, Integer enterpriseId, Integer loginType) {
        if (loginType == null) {
            loginType = LoginUserTypeEnum.GIC_USER.getCode();
        }
        int count = tabSysUserMapper.countByPhoneAndLoginType(phone, userId, enterpriseId, loginType);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public TabSysUser getUserByEnterpriseId(Integer enterpriseId) {
        return tabSysUserMapper.selectByEnterpriseId(enterpriseId);
    }

    @Override
    public List<TabSysUser> listAllUserByPhoneNumber(String phoneNumber, String nationCode) {
        return tabSysUserMapper.listUserByPhoneNumberAndEnterprise(phoneNumber, nationCode, null);
    }

    @Override
    public List<TabSysUser> listUserByPhoneNumberAndEnterprise(String phoneNumber, String nationCode,
            Integer enterpriseId) {
        return tabSysUserMapper.listUserByPhoneNumberAndEnterprise(phoneNumber, nationCode, enterpriseId);
    }

    @Override
    public TabSysUser login(String nationCode, String phoneNumber, Integer enterpriseId, String password) {
        return this.tabSysUserMapper.login(nationCode, phoneNumber, password, enterpriseId);
    }

    @Override
    public Page<UserListDTO> pageUser(UserListQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        List<UserListDTO> list = tabSysUserMapper.listUser(params);
        return (Page) list;
    }

    @Override
    public List<UserListDTO> listUser(UserListQO params) {
        return tabSysUserMapper.listUser(params);
    }

    @Override
    public void delete(Integer userId) {
        tabSysUserMapper.deleteById(userId);
    }

    @Override
    public Integer getAllCheckValue(Integer enterpriseId, List<Integer> userIdList) {
        Integer line = tabSysUserMapper.getFirstNotInUserId(enterpriseId, userIdList);
        return line == null ? GlobalInfo.DATA_STATUS_NORMAL : GlobalInfo.DATA_STATUS_DELETE;
    }

    @Override
    public boolean validUserIsNotExist(List<Integer> userIdList) {
        Integer count = tabSysUserMapper.countUserByUserIds(userIdList);
        return count != userIdList.size();
    }

    @Override
    public List<TabSysUser> listUserByIdList(List<Integer> userIdList) {
        return tabSysUserMapper.listUserByIdList(userIdList);
    }

    @Override
    public List<TabSysUser> listUserNotInIdList(List<Integer> userIdList, String search, Integer enterpriseId) {
        return tabSysUserMapper.listUserNotInIdList(userIdList, search, enterpriseId);
    }

    @Override
    public List<Integer> listAllUser(Integer enterpriseId) {
        return tabSysUserMapper.listAllUser(enterpriseId);
    }

    @Override
    public List<TabSysUser> listGicUser(Integer enterpriseId, Integer isGic) {
        return tabSysUserMapper.listGicUser(enterpriseId, isGic);
    }
}
