package com.gic.auth.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.constant.OrderOrAppEnum;
import com.gic.auth.constant.OrderResourceChannelEnum;
import com.gic.auth.constant.ResourceHoldEnum;
import com.gic.auth.dto.*;
import com.gic.auth.entity.TabSysOrderAppResource;
import com.gic.auth.entity.TabSysResource;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/28 18:28
 */
@Service("resourceApiService")
public class ResourceApiServiceImpl implements ResourceApiService {

    private Logger logger = LogManager.getLogger(ResourceApiServiceImpl.class);

    @Autowired
    private ResourceService resourceService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private OrderAppResourceService orderAppResourceService;
    @Autowired
    private UserService userService;
    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;


    @Override
    @Transactional
    public ServiceResponse<Integer> saveOrUpdateResource(ResourceDTO resourceDTO) {
        boolean repeat = resourceService.isRepeatByResourceName(resourceDTO.getEnterpriseId(), resourceDTO.getResourceName(), resourceDTO.getResourceId());
        if (repeat) {
            logger.info("enterpriseId:{} , resourceName:{}, 资源组名称重复 ", resourceDTO.getEnterpriseId(), resourceDTO.getResourceName());
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称重复");
        }
        OrderAppResourceDTO orderDTO = new OrderAppResourceDTO();
        orderDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        orderDTO.setType(OrderOrAppEnum.ORDER.getCode());
        orderDTO.setContent(resourceDTO.getOrderResourceJson());
        Integer order = orderAppResourceService.save(orderDTO);
        resourceDTO.setOrderResource(order);

        OrderAppResourceDTO appDTO = new OrderAppResourceDTO();
        appDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        appDTO.setType(OrderOrAppEnum.ORDER.getCode());
        appDTO.setContent(resourceDTO.getAppResourceJson());
        Integer app = orderAppResourceService.save(appDTO);
        resourceDTO.setAppResource(app);

        OrderAppResourceDTO smsSignDTO = new OrderAppResourceDTO();
        smsSignDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        smsSignDTO.setType(OrderOrAppEnum.SMS_SIGN.getCode());
        smsSignDTO.setContent(resourceDTO.getSmsSignResourceJson());
        Integer smsSign = orderAppResourceService.save(smsSignDTO);
        resourceDTO.setSmsSignResource(smsSign);

        OrderAppResourceDTO channelDTO = new OrderAppResourceDTO();
        channelDTO.setEnterpriseId(resourceDTO.getEnterpriseId());
        channelDTO.setType(OrderOrAppEnum.CHANNEL.getCode());
        channelDTO.setContent(resourceDTO.getChannelResourceJson());
        Integer channel = orderAppResourceService.save(channelDTO);
        resourceDTO.setChannelResource(channel);


        if (resourceDTO.getResourceId() == null) {
            // save
            resourceService.save(resourceDTO);
        } else {
            // update
            // 删除老的订单资源和应用资源
            TabSysResource resource = resourceService.getResource(resourceDTO.getResourceId());
            orderAppResourceService.del(resource.getOrderResource());
            orderAppResourceService.del(resource.getAppResource());
            orderAppResourceService.del(resource.getSmsSignResource());
            orderAppResourceService.del(resource.getChannelResource());
            resourceService.update(resourceDTO);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> delResource(Integer resourceId) {
        TabSysResource resource = resourceService.getResource(resourceId);
        boolean exist = this.userResourceService.existUserResource(resource.getEnterpriseId(), resource.getResourceId());
        if (exist) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该资源组正在使用中，不能删除");
        }
        Integer line = resourceService.delResource(resourceId);
        if (line > 0) {
            // 同步好办
            authorizeService.delResource(resourceId.longValue());
        }

        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<ResourceDTO>> listResource(ResourceDTO resourceDTO, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabSysResource> page = resourceService.listResource(resourceDTO, pageNum, pageSize);
        Page<ResourceDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, ResourceDTO.class);
        List<ResourceDTO> resourceList = resultPage.getResult();
        if (CollectionUtils.isNotEmpty(resourceList)) {
            List<Integer> resourceIds = resourceList.stream().map(ResourceDTO::getResourceId).collect(Collectors.toList());
            Map<Integer, Integer> map = userResourceService.countByResourceIds(resourceDTO.getEnterpriseId(), resourceIds);
            for (ResourceDTO resource : resourceList) {
                Integer count = map.get(resource.getResourceId());
                resource.setUserResourceCount(count == null ? 0 : count);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<ResourceDTO> getResource(Integer resourceId) {
        TabSysResource resource = resourceService.getResource(resourceId);
        if(resource == null){
            return ServiceResponse.success();
        }
        ResourceDTO resourceDTO = EntityUtil.changeEntityByOrika(ResourceDTO.class, resource);
        if (resourceDTO.getOrderResource() != null) {
            TabSysOrderAppResource orderResource = orderAppResourceService.getById(resourceDTO.getOrderResource());
            resourceDTO.setOrderResourceJson(orderResource.getContent());
        }
        if (resourceDTO.getAppResource() != null) {
            TabSysOrderAppResource appResource = orderAppResourceService.getById(resourceDTO.getAppResource());
            resourceDTO.setAppResourceJson(appResource.getContent());
        }
        if (resourceDTO.getSmsSignResource() != null) {
            TabSysOrderAppResource smsSignResource = orderAppResourceService.getById(resourceDTO.getSmsSignResource());
            resourceDTO.setSmsSignResourceJson(smsSignResource.getContent());
        }
        if (resourceDTO.getChannelResource() != null) {
            TabSysOrderAppResource channelResource = orderAppResourceService.getById(resourceDTO.getChannelResource());
            resourceDTO.setChannelResourceJson(channelResource.getContent());
        }
        return ServiceResponse.success(resourceDTO);
    }

    @Override
    public ServiceResponse<SmsSignResourceDTO> getSmsResourceByUserId(Integer userId) {
        TabSysUser user = userService.getUserById(userId);
        if (user.getSuperAdmin() == 1) {
            // 是超管就返回所有
            SmsSignResourceDTO superAdminResource = new SmsSignResourceDTO();
            superAdminResource.setType(1);
            return EnterpriseServiceResponse.success(superAdminResource);
        }


        List<UserResourceDTO> userResourceList = userResourceService.listUserResourceByUserId(userId);
        SmsSignResourceDTO defaultResource = new SmsSignResourceDTO();
        defaultResource.setType(2);
        defaultResource.setIdList(Collections.emptyList());
        if (CollectionUtils.isEmpty(userResourceList)) {
            return EnterpriseServiceResponse.success(defaultResource);
        }
        List<Integer> resourceIds = userResourceList.stream().map(UserResourceDTO::getResourceId).collect(Collectors.toList());
        List<TabSysResource> resourceList = resourceService.listByIds(resourceIds);
        List<Integer> smsSignList = resourceList.stream().filter(Objects::nonNull).map(TabSysResource::getSmsSignResource).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(smsSignList)) {
            return EnterpriseServiceResponse.success(defaultResource);
        }
        List<TabSysOrderAppResource> orderAppResourceList = orderAppResourceService.listByIds(smsSignList, OrderOrAppEnum.SMS_SIGN.getCode());
        SmsSignResourceDTO result = new SmsSignResourceDTO();
        List<Long> resultIds = new ArrayList<>();
        for (TabSysOrderAppResource orderAppResource : orderAppResourceList) {
            String json = orderAppResource.getContent();
            if (StringUtils.isNotBlank(json)) {
                JSONObject jsonObject = JSON.parseObject(json);
                Integer type = jsonObject.getInteger("type");
                if (type == 1) {
                    result.setType(1);
                    return EnterpriseServiceResponse.success(result);
                }
                String ids = jsonObject.getString("ids");
                if (StringUtils.isNotBlank(ids)) {
                    resultIds.addAll(Stream.of(ids.split(",")).map(Long::valueOf).collect(Collectors.toList()));
                }
            }
        }
        result.setIdList(resultIds);
        result.setType(2);
        return EnterpriseServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<ChannelResourceDTO> getChannelResourceByUserId(Integer userId) {

        TabSysUser user = this.userService.getUserById(userId);

        if (user == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }

        if (user.getSuperAdmin() == 1) {
            List<OwnerMemberCardDTO> memberCardList = memberCardApiService.getAll(user.getEnterpriseId()).getResult();
            List<AppletsConfigSimpleOutputDTO> appletList = appletsConfigApiService.listValidAppletConfigAppId(user.getEnterpriseId()).getResult();
            List<ServiceConfigSimpleOutputDTO> serviceList = serviceConfigApiService.listValidServiceConfigAppId(user.getEnterpriseId()).getResult();
            ChannelResourceDTO channelResourceDTO = new ChannelResourceDTO();

            channelResourceDTO.setAppletConfigIdList(Optional.ofNullable(appletList).orElse(Collections.emptyList()).stream().map(AppletsConfigSimpleOutputDTO::getAppletConfigId).collect(Collectors.toList()));
            channelResourceDTO.setServiceConfigIdList(Optional.ofNullable(serviceList).orElse(Collections.emptyList()).stream().map(ServiceConfigSimpleOutputDTO::getServiceConfigId).collect(Collectors.toList()));
            channelResourceDTO.setCardConfigIdList(Optional.ofNullable(memberCardList).orElse(Collections.emptyList()).stream().map(OwnerMemberCardDTO::getCardConfigId).collect(Collectors.toList()));

            return EnterpriseServiceResponse.success(channelResourceDTO);
        }


        List<UserResourceDTO> resourceDTOList = this.userResourceService.listUserResourceByUserId(userId);
        if (CollectionUtils.isEmpty(resourceDTOList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        UserResourceDTO userResourceDTO = resourceDTOList.get(0);
        TabSysResource resource = this.resourceService.getResource(userResourceDTO.getResourceId());

        if (resource.getChannelResource() == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);

        }
        TabSysOrderAppResource channelResource = orderAppResourceService.getById(resource.getChannelResource());

        ChannelResourceDTO channelResourceDTO = JSON.parseObject(channelResource.getContent(), ChannelResourceDTO.class);
        return EnterpriseServiceResponse.success(channelResourceDTO);
    }

    @Override
    public ServiceResponse<StoreResourceDTO> getStoreResourceByUserId(Integer userId) {
        TabSysUser user = userService.getUserById(userId);
        if (user.getSuperAdmin() == 1) {
            // 是超管就返回所有
            StoreResourceDTO superAdminResource = new StoreResourceDTO();
            superAdminResource.setType(ResourceHoldEnum.ALL.getCode());
            return EnterpriseServiceResponse.success(superAdminResource);
        }

        List<UserResourceDTO> userResourceList = userResourceService.listUserResourceByUserId(userId);
        StoreResourceDTO defaultResource = new StoreResourceDTO();
        defaultResource.setType(ResourceHoldEnum.DART.getCode());
        defaultResource.setStoreIdList(Collections.emptyList());
        if (CollectionUtils.isEmpty(userResourceList)) {
            return EnterpriseServiceResponse.success(defaultResource);
        }
        List<Integer> resourceIds = userResourceList.stream().map(UserResourceDTO::getResourceId).collect(Collectors.toList());
        List<TabSysResource> resourceList = resourceService.listByIds(resourceIds);
        List<Long> storeResourceList = resourceList.stream().filter(Objects::nonNull).map(TabSysResource::getStoreResource).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(storeResourceList)) {
            return EnterpriseServiceResponse.success(defaultResource);
        }
        HashSet<Integer> ids = new HashSet<>();
        for (Long storeResource : storeResourceList) {
            List<Integer> result = storeWidgetApiService.listStoreIdByStoreWidgetId(user.getEnterpriseId(), storeResource.intValue()).getResult();
            if (CollectionUtils.isNotEmpty(result)) {
                ids.addAll(result);
            }
        }
        List<Integer> result = new ArrayList<>(ids);
        defaultResource.setStoreIdList(result);
        return EnterpriseServiceResponse.success(defaultResource);
    }

    @Override
    public ServiceResponse<OrderResourceDTO> getOrderResourceByUserId(Integer userId) {
        TabSysUser user = userService.getUserById(userId);
        if (user.getSuperAdmin() == 1) {
            // 是超管就返回所有
            OrderResourceDTO superAdminResource = new OrderResourceDTO();
            superAdminResource.setType(ResourceHoldEnum.ALL.getCode());
            return EnterpriseServiceResponse.success(superAdminResource);
        }

        List<UserResourceDTO> resourceDTOList = this.userResourceService.listUserResourceByUserId(userId);
        if (CollectionUtils.isEmpty(resourceDTOList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        UserResourceDTO userResourceDTO = resourceDTOList.get(0);
        TabSysResource resource = this.resourceService.getResource(userResourceDTO.getResourceId());

        if (resource.getOrderResource() == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }

        OrderResourceDTO defaultResource = new OrderResourceDTO();
        defaultResource.setType(ResourceHoldEnum.DART.getCode());

        TabSysOrderAppResource sysOrderAppResource = orderAppResourceService.getById(resource.getOrderResource());

        List<OrderResourceContentDTO> contentDTOList = JSON.parseArray(sysOrderAppResource.getContent(), OrderResourceContentDTO.class);
        if (CollectionUtils.isNotEmpty(contentDTOList)) {
            for (OrderResourceContentDTO orderResourceContentDTO : contentDTOList) {
                if (OrderResourceChannelEnum.OFFLINE.getCode().equals(orderResourceContentDTO.getChannel())) {
                    List<Integer> result = storeWidgetApiService.listStoreIdByStoreWidgetId(user.getEnterpriseId(), orderResourceContentDTO.getStoreWidgetId()).getResult();
                    if (CollectionUtils.isNotEmpty(result)) {
                        defaultResource.setStoreIdList(result);
                    }
                } else if (OrderResourceChannelEnum.DAMO_MALL.getCode().equals(orderResourceContentDTO.getChannel())) {
                    defaultResource.setShopIdList(orderResourceContentDTO.getStoreContent());
                }
            }
        }

        return EnterpriseServiceResponse.success(defaultResource);
    }

    @Override
    public ServiceResponse<Long> getUserResourceByUserId(Integer userId) {
        TabSysUser user = userService.getUserById(userId);
        if (user.getSuperAdmin() == 1) {
            // 是超管就返回所有
            return EnterpriseServiceResponse.success(-1L);
        }
        List<UserResourceDTO> resourceDTOList = this.userResourceService.listUserResourceByUserId(userId);
        if (CollectionUtils.isEmpty(resourceDTOList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        UserResourceDTO userResourceDTO = resourceDTOList.get(0);
        TabSysResource resource = this.resourceService.getResource(userResourceDTO.getResourceId());
        return EnterpriseServiceResponse.success(resource.getUserResource());
    }
}
