package com.gic.auth.service.outer.impl;

import java.util.Date;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.service.UserResourceApiService;
import com.gic.auth.service.UserResourceService;
import com.gic.auth.service.UserService;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;

@Service("userResourceApiService")
public class UserResourceApiServiceImpl implements UserResourceApiService{
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private UserService userService;
    @Override
    public ServiceResponse<Void> save(UserResourceDTO dto) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, UserResourceDTO.SaveUserResourceValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        TabSysUser user = userService.getUserById(dto.getUserId());
        if (user == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户数据有误，查无数据");
        }

        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        dto.setStatus(1);

        userResourceService.save(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserResourceDTO> getResourceByUserId(Integer userId) {
        List<UserResourceDTO> list = this.userResourceService.listUserResourceByUserId(userId);
        if(CollectionUtils.isNotEmpty(list)){
            return ServiceResponse.success(list.get(0));
        }
        return ServiceResponse.success(new UserResourceDTO());
    }
}
