package com.gic.auth.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuthCodeDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.UserApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Calendar;
import java.util.Date;

@RestController
public class AuthCodeController {
    private static final Logger log = LogManager.getLogger(AuthCodeController.class);

    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;

    @RequestMapping("send-auth-code")
    public RestResponse sendAuthCode(Integer eid){
//        if(eid == null){
//            return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "商户不存在");
//        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        eid = enterpriseId;
        String code = CreateRandomUtils.getStringRandom(4);
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setAuthCode(code);
        authCodeDTO.setEnterpriseId(enterpriseId);
        authCodeDTO.setRelationId(eid);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.MINUTE, 5);
        authCodeDTO.setExpirationTime(calendar.getTime());
        //发送手机验证码
        ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseById(eid);
        if(enterprise.isSuccess() && enterprise.getResult() != null){
            String enterpriseName = UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName();
            ServiceResponse<UserDTO> user = this.userApiService.getUserByEnterpriseId(eid);
            String[] smsArr = new String[] { enterprise.getResult().getEnterpriseName(), enterpriseName, code, enterpriseName};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICUnionEnterprise",
                    UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(), user.getResult().getPhoneAreaCode(), user.getResult().getPhoneNumber(), smsArr);
            if(smsSendResult.isSuccess()){
                ServiceResponse<Integer> serviceResponse = this.authCodeApiService.saveAuth(authCodeDTO);
                return RestResponse.success(serviceResponse.getResult());
            }else {
                return RestResponse.failure(smsSendResult.getCode(), smsSendResult.getMessage());
            }
        }
        return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "商户不存在");
    }

    @RequestMapping("expire-auth-code")
    public RestResponse expireAuthCode(Integer authCodeId){
        ServiceResponse<Integer> serviceResponse = this.authCodeApiService.expireAuthCode(authCodeId);
        return RestResponse.success(serviceResponse.getResult());
    }

    @RequestMapping("get-auth-code")
    public RestResponse getAuthCode(Integer eid){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<AuthCodeDTO> authCode = this.authCodeApiService.getAuthCode(enterpriseId, eid);
        return RestResponse.success(authCode.getResult());
    }
}
