package com.gic.auth.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.dto.CollaboratorListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.qo.CollaboratorListQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.CollaboratorApiService;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.web.vo.CollaboratorDetailVO;
import com.gic.auth.web.vo.TestObj;
import com.gic.commons.util.DateUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.*;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.context.RequestContext;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@RestController
@RequestMapping("/collaborator")
public class CollaboratorController extends DownloadUtils{
    private static final Logger LOGGER = LogManager.getLogger(CollaboratorController.class);

    @Autowired
    private CollaboratorApiService collaboratorApiService;
    @Autowired
    private MenuApiService menuApiService;

    @RequestMapping("/list")
    public RestResponse list(CollaboratorListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(collaboratorApiService.page(params));
    }

    @RequestMapping("/list-app")
    public RestResponse listApp() {
        return ResultControllerUtils
                .commonResult(collaboratorApiService.listApp(UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("/download-test")
    public RestResponse downloadTest(CollaboratorListQO params, HttpServletRequest request, Integer excelExtensionCode) throws Exception {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        params.setEnterpriseId(userDetail.getEnterpriseId());
        //查询totalCount
        ServiceResponse<Page<CollaboratorListDTO>> result = collaboratorApiService.page(params);
        String fileName = "北丐测试";
        //生成报表中心记录
        DownloadReport report = new DownloadReport();
        report.setApplyUserId(userDetail.getUserInfo().getUserId());
        report.setDataContent("北丐-测试");
        report.setDataCount(result.getResult().getTotalCount());
        report.setDataType(2);
        //只适用于对接gic的登录拦截模块
        report.setDataUrl((String) request.getAttribute("moduleMenuUrl"));
        LOGGER.info("下载测试：{}", JSON.toJSONString(RequestContext.getContext().getRequest().getAttributeNames()));
        report.setEnterpriseId(userDetail.getEnterpriseId());
        report.setFileName(fileName);
        report.setReportEndTime(DateUtil.strToDate("2019-01-01 00:00:00", "yyyy-MM-dd HH:mm:ss"));
        report.setReportStartTime(DateUtil.strToDate("2018-01-01 00:00:00", "yyyy-MM-dd HH:mm:ss"));
        Integer reportId = DataDownloadUtils.createDownloadReport(report);
        //临时路径
        String path = request.getSession().getServletContext().getRealPath("/excel/csv/collaborator/file/");
        //异步生成文件并上传到腾讯云
        ExecutorPoolSingleton.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                List<Map<String, Object>> dataList = new ArrayList<>();
                Map<String, Object> map1 = new HashMap<>(8);
                map1.put("test1", "test1");
                map1.put("2", new TestObj("域名2门店1-1", "域名2门店2-1"));
                map1.put("3", new TestObj("域名3门店1-1", "域名3门店2-1"));
                dataList.add(map1);
                Map<String, Object> map2 = new HashMap<>(8);
                map2.put("test1", "test2");
                map2.put("2", new TestObj("域名2门店1-2", "域名2门店2-2"));
                map2.put("3", new TestObj("域名3门店1-2", "域名3门店2-2"));
                dataList.add(map2);
                DownloadDataLoader<Map<String, Object>> loader = new DownloadDataLoader<Map<String, Object>>() {
                    @Override
                    protected List<Map<String, Object>> getDownloadData(Integer pageNum) throws Exception {
                        params.setCurrentPage(pageNum);
//                        ServiceResponse<Page<CollaboratorListDTO>> result = collaboratorApiService.page(params);
                        if (pageNum == 2) {
                            //第二页结束test
                            return null;
                        }
                        return dataList;
                    }
                };
                LinkedHashMap<String, List<String>> headerList = new LinkedHashMap<>();
                headerList.put("测试1", null);
                headerList.put("域名2", Arrays.asList("门店1", "门店2"));
                headerList.put("域名3", Arrays.asList("门店1", "门店2"));
                LinkedHashMap<String, List<String>> propertyNameList = new LinkedHashMap<>();
                propertyNameList.put("test1", null);
                propertyNameList.put("2", Arrays.asList("store1", "store2"));
                propertyNameList.put("3", Arrays.asList("store1", "store2"));
                Map<String, List<String>> needEncryptField = new HashMap<>(16);
                needEncryptField.put("test1", null);
                needEncryptField.put("2", Arrays.asList("store1"));
                needEncryptField.put("3", Arrays.asList("store2"));
                try {
                    downloadForDoubleHeaderTitle(path, reportId, fileName, excelExtensionCode, headerList, propertyNameList, loader, needEncryptField, null);
                } catch (Exception e) {
                    LOGGER.warn(e);
                }
            }
        });
        return RestResponse.success(reportId);
    }

    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer collaboratorId) {
        LogUtils.createLog("获取协作人明细", "北丐测试");
        return ResultControllerUtils.commonResultOne(
                collaboratorApiService.getDetail(collaboratorId), CollaboratorDetailVO.class);
    }

    /**
     * 授权：操作授权、资源授权
     * @Title: auth 
     * @Description:
      * @author guojuxing
     * @param collaboratorId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/auth")
    public RestResponse auth(String subAppIds, String menuIds, String resourceIds, Integer collaboratorId) {
        ServiceResponse<String> result = collaboratorApiService.auth(subAppIds, menuIds, resourceIds, collaboratorId);
        String operationObject = result.isSuccess() ? result.getResult() : null;
        return OperationResultUtils.operationResult(result, "协作人授权", operationObject);
    }

    @RequestMapping("/app-menu-tree")
    public RestResponse appMenuTree(String appId) {
        MenuListQO params = new MenuListQO();
        params.setProject(appId);
        params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
        ServiceResponse<List<MenuDTO>> appMenuList = menuApiService.listMenuTree(params);
        if (appMenuList.isSuccess()) {
            return RestResponse.success(changePageListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, appMenuList.getResult()));
        }
        return EnterpriseRestResponse.failure(appMenuList);
    }

    public static List<Map<String, Object>> changePageListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                if (storeGroupId == menuDTO.getParentId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("menuCode", menuDTO.getMenuCode());
                    data.put("parentCode", menuDTO.getParentCode());
                    data.put("parentId", menuDTO.getParentId());
                    List<Map<String, Object>> children = changePageListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }
}
