package com.gic.auth.web.controller;

import com.gic.auth.web.vo.SuperUserVO;
import com.gic.auth.web.vo.UserOfGicVO;
import com.gic.enterprise.base.EnterpriseInfo;
import com.gic.enterprise.constants.Constants;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.UserConstants;
import com.gic.auth.dto.AuthCodeDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.utils.UserPasswordUtils;
import com.gic.auth.web.vo.LoginUserVO;
import com.gic.auth.web.vo.UserDetailVO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.Md5Util;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;

import java.util.Calendar;
import java.util.Date;

@RestController
@RequestMapping("/user")
public class UserController {

    private final Logger logger = LogManager.getLogger(UserController.class);

    @Autowired
    private UserApiService userApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;

    @RequestMapping("/save-or-update-user")
    public RestResponse editUser(@Validated({ UserDTO.UserQoValid.class }) UserDTO userDTO) {
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin().intValue() == 0) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "子管理员不能编辑");
        }
        Integer operationPassword = userDTO.getOperPasswordType();
        userDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());

        boolean isNeedSendSms = false;
        //发送短信的参数
        String[] smsArr = new String[0];
        if (operationPassword != null && operationPassword.intValue() == 1) {
            //如果是1，则不用验证密码

        } else {
            String password = UserPasswordUtils.validPassword(userDTO.getPasswordType(), userDTO.getPassword());
            userDTO.setPassword(UserPasswordUtils.getEncryptPassword(password));
            isNeedSendSms = UserConstants.CREATE_AUTO == userDTO.getPasswordType().intValue();
            if (isNeedSendSms) {
                //发送短信，通知自动生成密码
                if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                    userDTO.setPhoneAreaCode(Constants.NATION_CODE);
                }
                smsArr = new String[] { UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName(), password };
            }
        }
        ServiceResponse userResult;
        //普通用户
        userDTO.setSuperAdmin(0);

        //sms场景ID
        String smsSenceId = "GICSHHT003";
        if (userDTO.getUserId() == null) {
            userResult = userApiService.saveUser(userDTO);
        } else {
            smsSenceId = "GICSHHT004";
            userResult = userApiService.editUser(userDTO);
        }
        if (userResult.isSuccess()) {
            if (isNeedSendSms) {
                ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(smsSenceId,
                        UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(),
                        userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                if (!smsSendResult.isSuccess()) {
                    logger.warn(smsSendResult.getMessage());
                }
            }
            if (userDTO.getUserId() == null) {
                LogUtils.createLog(OperationResultUtils.LOG_NEW + "管理员", userDTO.getUserName());
            } else {
                LogUtils.createLog(OperationResultUtils.LOG_EDIT + "管理员", userDTO.getUserName());
            }
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(userResult);
        }
    }

    /**
     * 用户详情
     *
     * @param userId
     * @return com.gic.commons.webapi.reponse.RestResponse
     *
     * @Title: getUserById
     * @Description:   * @author guojuxing
     */
    @RequestMapping("/get-user-by-id")
    public RestResponse getUserById(Integer userId) {
        return ResultControllerUtils.commonResultOne(userApiService.getUserById(userId), UserDetailVO.class);
    }

    @RequestMapping("/delete-user-by-id")
    public RestResponse delete(Integer userId) {
        ServiceResponse<String> result = userApiService.delete(userId);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "管理员",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-user")
    public RestResponse listUser(UserListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.pageUser(params));
    }

    /**
     * 个人资料
     * @Title: getLoginUser 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/get-login-user")
    public RestResponse getLoginUser() {
        LoginUserVO vo = EntityUtil.changeEntityNew(LoginUserVO.class, UserDetailUtils.getUserDetail().getUserInfo());
        //商户logo
        EnterpriseInfo enterpriseInfo = UserDetailUtils.getUserDetail().getEnterpriseInfo();
        vo.setLogo(enterpriseInfo.getLogo());
        vo.setEnterpriseName(enterpriseInfo.getEnterpriseName());
        vo.setCompanyName(enterpriseInfo.getCompanyName());
        return RestResponse.success(vo);
    }

    @RequestMapping("send-auth-code-to-modify-phone")
    public RestResponse sendAuthCode(String phone, String nationCode) {
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //生成4位数字验证码
        String authCode = CreateRandomUtils.getStringRandom(4);
        authCodeDTO.setAuthCode(authCode);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.MINUTE, 5);
        authCodeDTO.setExpirationTime(calendar.getTime());
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] { authCode };
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICSHHT005",
                    UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(), nationCode, phone, smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("modify-phone")
    public RestResponse modifyPhone(String phone, String nationCode, String authCode, Integer authCodeId) {
        ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(authCodeId, authCode);
        if (authCodeResult.isSuccess()) {
            UserDetail userDetail = UserDetailUtils.getUserDetail();
            // password 自身作为盐值
            String password = userDetail.getUserInfo().getPassword();
            ServiceResponse<UserDTO> login = this.userApiService.login(nationCode,
                    userDetail.getUserInfo().getPhoneNumber(), userDetail.getEnterpriseId(), password);
            ServiceResponse<String> result = userApiService.editPhoneOrPassword(login.getResult().getUserId(),
                    nationCode, phone, null);
            if (result.isSuccess()) {
                //过期验证码
                authCodeApiService.expireAuthCode(authCodeId);
            }
            return OperationResultUtils.operationResult(result, "修改手机号",
                    OperationResultUtils.getOperationObject(result));
        }
        return EnterpriseRestResponse.failure(authCodeResult);
    }

    @RequestMapping("send-auth-code-to-modify-password")
    public RestResponse sendAuthCode() {
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //生成4位数字验证码
        String authCode = CreateRandomUtils.getStringRandom(4);
        authCodeDTO.setAuthCode(authCode);
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] {
                    UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName(), authCode };
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICSHHT004",
                    UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(),
                    UserDetailUtils.getUserDetail().getUserInfo().getPhoneAreaCode(),
                    UserDetailUtils.getUserDetail().getUserInfo().getPhoneNumber(), smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("modify-password")
    public RestResponse modifyPassword(String oldPassword, String newPassword) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        Md5Util md5 = new Md5Util();
        // password 自身作为盐值
        if (oldPassword.length() != 32) {
            oldPassword = md5.encrypt(oldPassword + oldPassword);
        }
        ServiceResponse<UserDTO> login = this.userApiService.login(userDetail.getUserInfo().getPhoneAreaCode(),
                userDetail.getUserInfo().getPhoneNumber(), userDetail.getEnterpriseId(), oldPassword);
        if (!login.isSuccess()) {
            return EnterpriseRestResponse.failure(login);
        }
        if (login.getResult() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "旧密码错误");
        }
        ServiceResponse<String> result = userApiService.editPhoneOrPassword(login.getResult().getUserId(), null, null,
                md5.encrypt(newPassword + newPassword));
        return OperationResultUtils.operationResult(result, "修改密码", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-user-nopage")
    public RestResponse listUserNoPage(UserListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.listUser(params));
    }

    @RequestMapping("/list-gic-user")
    public RestResponse listGicUser() {
        return ResultControllerUtils.commonResult(userApiService.listGicUser(UserDetailUtils.getUserDetail().getEnterpriseId()), UserOfGicVO.class);
    }

    /**
     * 逻辑删除用户对应分组数据
     *
     * @param type
     * @param accountGroupId
     * @param userId
     * @return com.gic.commons.webapi.reponse.RestResponse
     *
     * @Title: removeUserAccountGroup
     * @Description:   * @author guojuxing
     */
    @RequestMapping("/remove-user-account-group")
    public RestResponse removeUserAccountGroup(Integer type, Integer accountGroupId, Integer userId) {
        ServiceResponse<String> result = userApiService.removeAccountGroup(accountGroupId, userId, type);
        return OperationResultUtils.operationResult(result, "移除账号分组", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/get-super-user")
    public RestResponse getSuperAdmin() {
        return ResultControllerUtils.commonResultOne(
                userApiService.getUserByEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId()),
                SuperUserVO.class);
    }

}
