package com.gic.auth.web.utils;

import com.gic.store.constant.Constants;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zhiwj
 * @date 2019/7/2
 */
public class ExcelUtils {

    private static final short ROW_HEIGHT = 500;
    private static final int COLUMN_HEIGHT = 5000;

    public static XSSFWorkbook getXSSFWorkbook(String sheetName, List<List<String>> everyLines, XSSFWorkbook wb) {
        if (CollectionUtils.isEmpty(everyLines)) {
            return null;
        }
        // 第一步，创建一个XSSFWorkbook，对应一个Excel文件
        if (wb == null) {
            wb = new XSSFWorkbook();
        }


        // 第二步，在workbook中添加一个sheet,对应Excel文件中的sheet
        XSSFSheet sheet = wb.createSheet(sheetName);

        sheet.setDefaultRowHeight(ROW_HEIGHT);
        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
        XSSFRow row = sheet.createRow(0);
        // 第四步，创建单元格，并设置值表头
        XSSFCellStyle style = wb.createCellStyle();
        XSSFFont font = wb.createFont();
        font.setFontName("微软雅黑");
        style.setFont(font);
        //声明列对象
        XSSFCell cell = null;

        //创建标题
        List<String> titles = everyLines.get(0);
        for (int i = 0; i < titles.size(); i++) {
            cell = row.createCell(i);
            cell.setCellValue(titles.get(i));
            cell.setCellStyle(style);
        }

        //创建内容
        for (int i = 1; i < everyLines.size(); i++) {
            row = sheet.createRow(i);
            for (int j = 0; j < everyLines.get(i).size(); j++) {
                sheet.setColumnWidth(j, COLUMN_HEIGHT);
                //将内容按顺序赋给对应的列对象
                XSSFCell cell1 = row.createCell(j);
                cell1.setCellValue(everyLines.get(i).get(j));
                cell1.setCellStyle(style);
            }
        }
        return wb;
    }


    public static Workbook getWorkbook(String title, List<String> rowName, List<String> description, List<List<String>> dataList, String regionIdStr) {

        XSSFWorkbook workbook = new XSSFWorkbook(); // 创建工作簿对象
        XSSFSheet sheet = workbook.createSheet(title); // 创建工作表

        XSSFRow row1 = sheet.createRow(0);
        row1.setZeroHeight(true);
        XSSFCell cell1 = row1.createCell(0);
        if (StringUtils.isNotBlank(regionIdStr)) {
            cell1.setCellValue(regionIdStr);
        }

        XSSFRow rowm = sheet.createRow(1);  // 产生表格标题行
        XSSFCell cellTiltle = rowm.createCell(0);   //创建表格标题列
        // sheet样式定义;    getColumnTopStyle();    getStyle()均为自定义方法 --在下面,可扩展
        XSSFCellStyle columnTopStyle = ExcelUtils.getColumnTopStyle(workbook);// 获取列头样式对象
        XSSFCellStyle titleStyle = ExcelUtils.getTitleStyle(workbook);// 获取列头样式对象
        XSSFCellStyle style = ExcelUtils.getStyle(workbook); // 获取单元格样式对象
        //合并表格标题行，合并列数为列名的长度,第一个0为起始行号，第二个1为终止行号，第三个0为起始列好，第四个参数为终止列号
        sheet.addMergedRegion(new CellRangeAddress(1, 2, 0, (rowName.size() - 1)));
        cellTiltle.setCellStyle(columnTopStyle);    //设置标题行样式
        cellTiltle.setCellValue(title);     //设置标题行值
        int columnNum = rowName.size();     // 定义所需列数
        XSSFRow rowRowName = sheet.createRow(3); // 在索引2的位置创建行(最顶端的行开始的第二行)
        // 将列头设置到sheet的单元格中
        for (int n = 0; n < columnNum; n++) {
            XSSFCell cellRowName = rowRowName.createCell(n); // 创建列头对应个数的单元格
            cellRowName.setCellType(XSSFCell.CELL_TYPE_STRING); // 设置列头单元格的数据类型
            XSSFRichTextString text = new XSSFRichTextString(rowName.get(n));
            cellRowName.setCellValue(text); // 设置列头单元格的值
            cellRowName.setCellStyle(titleStyle); // 设置列头单元格样式
        }
        // 定义数据开始的行数
        int dataLine = 4;
        if (CollectionUtils.isNotEmpty(description)) {
            int descriptionNum = description.size();     // 定义所需列数
            XSSFRow descriptionRow = sheet.createRow(dataLine); // 在索引2的位置创建行(最顶端的行开始的第二行)
            dataLine++;
            for (int n = 0; n < descriptionNum; n++) {
                XSSFCell cellRowName = descriptionRow.createCell(n); // 创建列头对应个数的单元格
                cellRowName.setCellType(XSSFCell.CELL_TYPE_STRING); // 设置列头单元格的数据类型
                XSSFRichTextString text = new XSSFRichTextString(description.get(n));
                cellRowName.setCellValue(text); // 设置列头单元格的值
                cellRowName.setCellStyle(titleStyle); // 设置列头单元格样式
            }
        }

        // 将查询出的数据设置到sheet对应的单元格中
        for (int i = 0; i < dataList.size(); i++) {
            List<String> obj = dataList.get(i);   // 遍历每个对象
            XSSFRow row = sheet.createRow(i + dataLine);   // 创建所需的行数
            for (int j = 0; j < obj.size(); j++) {
                XSSFCell cell = null;   // 设置单元格的数据类型
                if (j == 0) {
                    cell = row.createCell(j, XSSFCell.CELL_TYPE_NUMERIC);
                    cell.setCellValue(i + 1);
                } else {
                    cell = row.createCell(j, XSSFCell.CELL_TYPE_STRING);
                    if (!"".equals(obj.get(j)) && obj.get(j) != null) {
                        cell.setCellValue(obj.get(j)); // 设置单元格的值
                    }
                }
                cell.setCellStyle(style); // 设置单元格样式
            }
        }

        // 让列宽随着导出的列长自动适应
        for (int colNum = 0; colNum < columnNum; colNum++) {
            int columnWidth = sheet.getColumnWidth(colNum) / 256;
            for (int rowNum = 0; rowNum < sheet.getLastRowNum(); rowNum++) {
                XSSFRow currentRow;
                // 当前行未被使用过
                if (sheet.getRow(rowNum) == null) {
                    currentRow = sheet.createRow(rowNum);
                } else {
                    currentRow = sheet.getRow(rowNum);
                }
                if (currentRow.getCell(colNum) != null) {
                    XSSFCell currentCell = currentRow.getCell(colNum);
                    if (currentCell.getCellType() == XSSFCell.CELL_TYPE_STRING) {
                        int length = currentCell.getStringCellValue()
                                .getBytes().length;
                        if (columnWidth < length) {
                            columnWidth = length;
                        }
                    }
                }
            }
            if (colNum == 0) {
                sheet.setColumnWidth(colNum, (columnWidth - 2) * 256);
            } else {
                sheet.setColumnWidth(colNum, (columnWidth + 4) * 256);
            }
        }
        return workbook;
    }

    private static XSSFCellStyle getTitleStyle(XSSFWorkbook workbook) {
        XSSFCellStyle style = ExcelUtils.getColumnTopStyle(workbook);
        style.setFillForegroundColor(IndexedColors.LIGHT_GREEN.getIndex());
//        style.setFillPattern(CellStyle.SOLID_FOREGROUND);
        return style;
    }

    /*
     * 列头单元格样式
     */
    private static XSSFCellStyle getColumnTopStyle(XSSFWorkbook workbook) {

        // 设置字体
        XSSFFont font = workbook.createFont();
        // 设置字体大小
        font.setFontHeightInPoints((short) 11);
        // 字体加粗
//        font.setBoldweight(XSSFFont.BOLDWEIGHT_BOLD);
        // 设置字体名字
        font.setFontName("Courier New");
        // 设置样式;
        XSSFCellStyle style = workbook.createCellStyle();
        // 设置底边框;
//        style.setBorderBottom(XSSFCellStyle.BORDER_THIN);
        // 设置底边框颜色;
//        style.setBottomBorderColor(XSSFColor.BLACK.index);
        // 设置左边框;
//        style.setBorderLeft(XSSFCellStyle.BORDER_THIN);
        // 设置左边框颜色;
//        style.setLeftBorderColor(XSSFColor.BLACK.index);
        // 设置右边框;
//        style.setBorderRight(XSSFCellStyle.BORDER_THIN);
        // 设置右边框颜色;
//        style.setRightBorderColor(XSSFColor.BLACK.index);
        // 设置顶边框;
//        style.setBorderTop(XSSFCellStyle.BORDER_THIN);
        // 设置顶边框颜色;
//        style.setTopBorderColor(XSSFColor.BLACK.index);
        // 在样式用应用设置的字体;
        style.setFont(font);
        // 设置自动换行;
        style.setWrapText(false);
        // 设置水平对齐的样式为居中对齐;
//        style.setAlignment(XSSFCellStyle.ALIGN_CENTER);
        // 设置垂直对齐的样式为居中对齐;
//        style.setVerticalAlignment(XSSFCellStyle.VERTICAL_CENTER);

        return style;

    }

    /*
     * 列数据信息单元格样式
     */
    private static XSSFCellStyle getStyle(XSSFWorkbook workbook) {

        // 设置字体
        XSSFFont font = workbook.createFont();
        // 设置字体大小
        // font.setFontHeightInPoints((short)10);
        // 字体加粗
        // font.setBoldweight(XSSFFont.BOLDWEIGHT_BOLD);
        // 设置字体名字
        font.setFontName("Courier New");
        font.setColor(IndexedColors.RED.index);
        // 设置样式;
        XSSFCellStyle style = workbook.createCellStyle();
        // 设置底边框;
//        style.setBorderBottom(XSSFCellStyle.BORDER_THIN);
        // 设置底边框颜色;
//        style.setBottomBorderColor(XSSFColor.BLACK.index);
        // 设置左边框;
//        style.setBorderLeft(XSSFCellStyle.BORDER_THIN);
        // 设置左边框颜色;
//        style.setLeftBorderColor(XSSFColor.BLACK.index);
        // 设置右边框;
//        style.setBorderRight(XSSFCellStyle.BORDER_THIN);
        // 设置右边框颜色;
//        style.setRightBorderColor(XSSFColor.BLACK.index);
        // 设置顶边框;
//        style.setBorderTop(XSSFCellStyle.BORDER_THIN);
        // 设置顶边框颜色;
//        style.setTopBorderColor(XSSFColor.BLACK.index);
        // 在样式用应用设置的字体;
        style.setFont(font);
        // 设置自动换行;
        style.setWrapText(false);
        // 设置水平对齐的样式为居中对齐;
//        style.setAlignment(XSSFCellStyle.ALIGN_CENTER);
        // 设置垂直对齐的样式为居中对齐;
//        style.setVerticalAlignment(XSSFCellStyle.VERTICAL_CENTER);

        return style;
    }

    public static Workbook getWorkbookFromUpload(InputStream inputStream, String fileName) throws IOException {
        Workbook wb;
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        if (".xlsx".equals(fileType)) {
            wb = new XSSFWorkbook(inputStream);
        } else if ("xls".equals(fileType)) {
            wb = new HSSFWorkbook(inputStream);
        } else {
            return null;
        }
        return wb;
    }


    /**
     * @param row      表示数据从第几行开始 例如：第一行为表头，第二行为数据，row的值为2
     * @param fileType 文件的后缀 例如：xls xlsx
     * @return
     */
    public static List<Map<Integer, String>> readExcel(int row, String fileType, Workbook xssfWorkbook, Map<Integer, Boolean> dateTypeMap) {
        if (fileType.equals("xlsx")) {
            return readXlsx(xssfWorkbook, row, dateTypeMap);
        } else {
            return null;
        }
    }

    public static List<Map<Integer, String>> readXlsx(Workbook xssfWorkbook, int row, Map<Integer, Boolean> dateTypeMap) {
        List<Map<Integer, String>> result = new ArrayList<Map<Integer, String>>();
        SimpleDateFormat format = new SimpleDateFormat(Constants.dateformat);
//            XSSFWorkbook xssfWorkbook = new XSSFWorkbook(in);
        for (int numSheet = 0; numSheet < 1; numSheet++) {
            Sheet xssfSheet = xssfWorkbook.getSheetAt(numSheet);
            if (xssfSheet == null || xssfSheet.getLastRowNum() == 1) {
                continue;
            }
            for (int rowNum = row - 1; rowNum <= xssfSheet.getLastRowNum(); rowNum++) {
                Row xssfRow = xssfSheet.getRow(rowNum);
                if (xssfRow == null) {
                    continue;
                }
                int minColix = xssfRow.getFirstCellNum();
                int maxColix = xssfRow.getLastCellNum();

                Map<Integer, String> map = new HashMap<Integer, String>();
                for (int colix = minColix; colix < maxColix; colix++) {
                    Cell xssfCell = xssfRow.getCell(colix);

                    if (xssfCell == null) {
                        map.put(colix, "");
                    } else {

                        Boolean isDate = dateTypeMap.get(colix);
                        if (isDate != null && isDate) {
                            if (xssfCell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
                                Date date = xssfCell.getDateCellValue();
                                map.put(colix, format.format(date));
                            } else {
                                xssfCell.setCellType(Cell.CELL_TYPE_STRING);
                                map.put(colix, xssfCell.getStringCellValue());
                            }
                        } else {
                            if (xssfCell.getCellType() != Cell.CELL_TYPE_STRING) {
                                xssfCell.setCellType(Cell.CELL_TYPE_STRING);
                            }
                            map.put(colix, xssfCell.getStringCellValue());
                        }
                    }
                }
                if (!isEmpty(map, minColix, maxColix)) {
                    result.add(map);
                }
            }
        }

        return result;
    }

    private static boolean isEmpty(Map<Integer, String> data, int startIndex, int endIndex) {
        boolean isEmplty = true;
        for (int i = startIndex + 1; i <= endIndex; i++) {
            String value = data.get(i);
            if (!StringUtils.isBlank(value)) {
                isEmplty = false;
            }
        }
        return isEmplty;
    }
}